@@ -1,6 +1,6 @@
 package Getopt::EX::Hashed;
 
-our $VERSION = '0.9912';
+our $VERSION = '0.9913';
 
 =head1 NAME
 
@@ -8,7 +8,7 @@ Getopt::EX::Hashed - Hash store object automation
 
 =head1 VERSION
 
-Version 0.9912
+Version 0.9913
 
 =head1 SYNOPSIS
 
@@ -18,9 +18,10 @@ Version 0.9912
   package App::foo;
 
   use Getopt::EX::Hashed;
-  has start => ( spec => "=i s begin", default => 1 );
-  has end   => ( spec => "=i e" );
-  has file  => ( spec => "=s", is => 'rw' );
+  has start  => ( spec => "=i s begin", default => 1 );
+  has end    => ( spec => "=i e" );
+  has file   => ( spec => "=s", is => 'rw' );
+  has answer => ( spec => '=i', must => sub { $_[1] == 42 } );
   no  Getopt::EX::Hashed;
 
   sub run {
@@ -55,6 +56,7 @@ my %DefaultConfig = (
     DEBUG_PRINT        => 0,
     LOCK_KEYS          => 1,
     REPLACE_UNDERSCORE => 1,
+    REMOVE_UNDERSCORE  => 0,
     RESET_AFTER_NEW    => 0,
     GETOPT             => 'GetOptions',
     ACCESSOR_PREFIX    => '',
@@ -68,11 +70,11 @@ sub import {
     my $caller = caller;
     no strict 'refs';
     push @{"$caller\::ISA"}, __PACKAGE__;
-    *{"$caller\::$_"} = \&{$_} for @EXPORT;
+    *{"$caller\::$_"} = \&$_ for @EXPORT;
     my $C = __Config__($caller);
-    unless (%{$C}) {
-	%{$C} = %DefaultConfig or die "something wrong!";
-	lock_keys %{$C};
+    unless (%$C) {
+	%$C = %DefaultConfig or die "something wrong!";
+	lock_keys %$C;
     }
 }
 
@@ -94,7 +96,7 @@ sub unimport {
 
 sub reset {
     my $M = __Member__(caller);
-    @{$M} = ();
+    @$M = ();
     return $_[0];
 }
 
@@ -106,18 +108,18 @@ sub has {
     my $C = __Config__($caller);
     for my $name (@name) {
 	my $append = $name =~ s/^\+//;
-	my $i = first { ${$M}[$_]->[0] eq $name } 0 .. $#{$M};
+	my $i = first { $M->[$_]->[0] eq $name } 0 .. $#{$M};
 	if ($append) {
 	    defined $i or die "$name: Not found\n";
-	    push @{${$M}[$i]}, @param;
+	    push @{$M->[$i]}, @param;
 	} else {
 	    defined $i and die "$name: Duplicated\n";
 	    if (my $default = $C->{DEFAULT}) {
 		if (ref $default eq 'ARRAY') {
-		    unshift @param, @{$default};
+		    unshift @param, @$default;
 		}
 	    }
-	    push @{$M}, [ $name, @param ];
+	    push @$M, [ $name, @param ];
 	}
     }
 }
@@ -132,10 +134,10 @@ sub new {
 	map {
 	    my($key, %param) = @$_;
 	    $key => \%param;
-	} @{$M}
+	} @$M
     };
-    my $order = $obj->{__Order__} = [ map $_->[0], @{$M} ];
-    for my $key (@{$order}) {
+    my $order = $obj->{__Order__} = [ map $_->[0], @$M ];
+    for my $key (@$order) {
 	my $m = $member->{$key};
 	$obj->{$key} = $m->{default};
 	if (my $is = $m->{is}) {
@@ -144,7 +146,7 @@ sub new {
 	    *{"$class\::$access"} = _accessor($is, $key);
 	}
     }
-    lock_keys %{$obj} if $C->{LOCK_KEYS};
+    lock_keys %$obj if $C->{LOCK_KEYS};
     __PACKAGE__->reset if $C->{RESET_AFTER_NEW};
     $obj;
 }
@@ -154,11 +156,11 @@ sub _accessor {
     {
 	ro => sub {
 	    $#_ and die "$name is readonly\n";
-	    $_[0]{$name};
+	    $_[0]->{$name};
 	},
 	rw => sub {
-	    $#_ and do { $_[0]{$name} = $_[1]; return $_[0] };
-	    $_[0]{$name};
+	    $#_ and do { $_[0]->{$name} = $_[1]; return $_[0] };
+	    $_[0]->{$name};
 	}
     }->{$is} or die "$name has invalid 'is' parameter.\n";
 }
@@ -174,26 +176,7 @@ sub _optspec {
     my $obj = shift;
     my $ctx = shift;
     my $member = $obj->{__Hash__};
-    my @optlist = do {
-	map  {
-	    my($name, $spec) = @$_;
-	    my $compiled = _compile($ctx, $name, $spec);
-	    my $m = $member->{$name};
-	    my $dest = do {
-		if (my $action = $m->{action}) {
-		    ref $action eq 'CODE' or
-			die "$name: action must be coderef.\n";
-		    sub { &$action for $obj };
-		} else {
-		    if (ref $obj->{$name} eq 'CODE') {
-			sub { &{$obj->{$name}} for $obj };
-		    } else {
-			\$obj->{$name};
-		    }
-		}
-	    };
-	    $compiled => $dest;
-	}
+    my @spec = do {
 	# spec .= alias
 	map  {
 	    if (my $alias = $member->{$_->[0]}->{alias}) {
@@ -210,7 +193,44 @@ sub _optspec {
 	map  { [ $_ => $member->{$_}->{spec} ] }
 	@{$obj->{__Order__}};
     };
-    @optlist;
+    my @optlist = map {
+	my($name, $spec) = @$_;
+	my $compiled = _compile($ctx, $name, $spec);
+	my $m = $member->{$name};
+	my $action = $m->{action};
+	$action and ref $action ne 'CODE'
+	    and die "$name->action: not a coderef.\n";
+	my $dest = do {
+	    if (my $must = $m->{must}) {
+		ref $must ne 'CODE'
+		    and die "$name->must: not a coderef.\n";
+		$action ||= \&_generic_setter;
+		sub {
+		    local $_ = $obj;
+		    &$must or die "@_: invalid value.\n";
+		    &$action;
+		};
+	    }
+	    elsif ($action) {
+		sub { &$action for $obj };
+	    }
+	    else {
+		if (ref $obj->{$name} eq 'CODE') {
+		    sub { &{$obj->{$name}} for $obj };
+		} else {
+		    \$obj->{$name};
+		}
+	    }
+	};
+	$compiled => $dest;
+    } @spec;
+}
+
+sub _generic_setter {
+    my $dest = $_->{$_[0]};
+    (ref $dest eq 'ARRAY') ? do { push @$dest, $_[1] } :
+    (ref $dest eq 'HASH' ) ? do { $dest->{$_[1]} = $_[2] }
+                           : do { $_->{$_[0]} = $_[1] };
 }
 
 my $spec_re = qr/[!+=:]/;
@@ -231,10 +251,9 @@ sub _compile {
     my @alias = grep !/$spec_re/, @args;
     my @names = ($name, @alias);
     my $C = __Config__($ctx);
-    if ($C->{REPLACE_UNDERSCORE}) {
-	for ($name, @alias) {
-	    push @names, tr[_][-]r if /_/;
-	}
+    for ($name, @alias) {
+	push @names, tr[_][-]r if /_/ && $C->{REPLACE_UNDERSCORE};
+	push @names, tr[_][]dr if /_/ && $C->{REMOVE_UNDERSCORE};
     }
     push @names, '' if @names and $spec !~ /^($spec_re|$)/;
     join('|', @names) . $spec;
@@ -247,7 +266,7 @@ sub getopt {
     my $C = __Config__($ctx);
     my $getopt = caller . "::" . $C->{GETOPT};
     no strict 'refs';
-    &{$getopt}(_optspec($obj, $ctx));
+    $getopt->(_optspec($obj, $ctx));
 }
 
 sub use_keys {
@@ -363,17 +382,17 @@ In fact, B<default> parameter takes code reference too.  It is stored
 in the hash object and the code works almost same.  But the hash value
 can not be used for option storage.
 
-Because B<action> function intercept the option assignment, it can be
-used to verify the parameter.
+=item B<must> => I<coderef>
 
-    has age =>
+Parameter B<must> takes a code reference to validate option values.
+It takes same arguments as B<action> and returns boolean.  With next
+example, option B<--answer> takes only 42 as a valid value.
+
+    has answer =>
         spec => '=i',
-        action => sub {
-            my($name, $i) = @_;
-            (0 <= $i and $i <= 150) or
-                die "$name: have to be in 0 to 150 range.\n";
-            $_->{$name} = $i;
-        };
+        must => sub { $_[1] == 42 };
+
+Can be used with B<action> parameter.
 
 =back
 
